<?php

namespace Drupal\webformautosave;

use Drupal\webform\WebformSubmissionInterface;
use Drupal\webform_submission_log\WebformSubmissionLogManager;
use stdClass;

/**
 * Class AutosaveHelper.
 *   Houses helper functions for the Webform Autosave module.
 *
 * @package Drupal\webformautosave
 */
class AutosaveHelper {

  /**
   * @var \Drupal\webform_submission_log\WebformSubmissionLogManager
   */
  protected $webform_submission_log_manager;

  /**
   * AutosaveHelper constructor.
   *
   * @param \Drupal\webform_submission_log\WebformSubmissionLogManager $webform_submission_log_manager
   */
  public function __construct(WebformSubmissionLogManager $webform_submission_log_manager) {
    $this->webform_submission_log_manager = $webform_submission_log_manager;
  }

    /**
   * Getter for the most recent submission log.
   *
   * @param \Drupal\webform\WebformSubmissionInterface $webform_submission
   *
   * @return \Drupal\Core\Url
   */
  public function getSubmissionUrl(WebformSubmissionInterface $webform_submission) {
    /** @var \Drupal\Core\Url $submission_url */
    $submission_url = $webform_submission->getTokenUrl();
    $submission_url->setAbsolute(FALSE);
    $current_params = (array) \Drupal::request()->query->all();
    // Add the current params to the submission url.
    foreach ($current_params as $key => $param) {
      $submission_url->setRouteParameter($key, $param);
    }
    return $submission_url;
  }

  /**
   * Getter for the most recent submission log.
   *
   * @param \Drupal\webform\WebformSubmissionInterface $webform_submission
   *
   * @return mixed
   */
  public function getCurrentSubmissionLog(WebformSubmissionInterface $webform_submission) {
    $submission_logs = $this->webform_submission_log_manager->loadByEntities($webform_submission->getWebform(), $webform_submission->getSourceEntity());
    if (!empty($submission_logs)) {
      // Return the last log object in the array.
      return end($submission_logs);
    }
    else {
      // Return a vanilla log with the current timestamp and UID.
      $submission_log = new stdClass();
      $submission_log->timestamp = \Drupal::time()->getRequestTime();;
      $submission_log->uid = \Drupal::currentUser()->id();
      return $submission_log;
    }
  }

  /**
   * Getter for all the fields on the current page.
   *
   * @param \Drupal\webform\WebformSubmissionInterface $webform_submission
   *
   * @return array|bool
   */
  public function getCurrentFields(WebformSubmissionInterface $webform_submission) {
    $webform = $webform_submission->getWebform();
    $elements = $webform->getElementsInitializedAndFlattened();
    $is_wizzard = $webform->hasWizardPages();
    if ($is_wizzard && is_array($elements)) {
      $current_page = $webform_submission->getCurrentPage();
      return $elements[$current_page]['#webform_children'];
    }
    elseif (is_array($elements)) {
      return $elements;
    }
    return FALSE;
  }

  /**
   * Getter for the first field on the current page.
   *
   * @param \Drupal\webform\WebformSubmissionInterface $webform_submission
   *
   * @return string|bool
   */
  public function getFirstWebformField(WebformSubmissionInterface $webform_submission) {
    $webform = $webform_submission->getWebform();
    $elements = $this->getCurrentFields($webform_submission);
    $is_wizzard = $webform->hasWizardPages();
    if ($is_wizzard && is_array($elements)) {
      return key($elements);
    }
    elseif (is_array($elements)) {
      return key($elements);
    }
    return FALSE;
  }

  /**
   * Getter for the first field on the next page.
   *
   * @param \Drupal\webform\WebformSubmissionInterface $webform
   *   Webform interface.
   *
   * @return string|bool
   */
  public function getNextPageFirstWebformField(WebformSubmissionInterface $webform_submission) {
    $webform = $webform_submission->getWebform();
    $elements = $webform->getElementsInitializedAndFlattened();
    $is_wizzard = $webform->hasWizardPages();
    if ($is_wizzard && is_array($elements)) {
      $current_page = $webform_submission->getCurrentPage();
      $pages = array_keys($webform->getPages());
      $current_index = array_search($current_page, $pages);
      $next_page = $pages[$current_index + 1];
      if (array_key_exists($next_page, $elements)) {
        return key($elements[$next_page]['#webform_children']);
      }
    }
    return FALSE;
  }

}
