<?php

namespace Drupal\webformautosave\Plugin\rest\resource;

use Drupal\webform\Entity\WebformSubmission;
use Drupal\rest\Plugin\ResourceBase;
use Drupal\rest\ModifiedResourceResponse;
use Drupal\webform\WebformSubmissionInterface;
use Symfony\Component\HttpKernel\Exception\NotFoundHttpException;

/**
 * Creates a resource for retrieving webform submission data.
 *
 * @RestResource(
 *   id = "webformautosave_submission",
 *   label = @Translation("Webform Autosave Submission"),
 *   uri_paths = {
 *     "canonical" = "/webformautosave/{webform_id}/submission/{sid}"
 *   }
 * )
 */
class WebformSubmissionResource extends ResourceBase {

  /**
   * Retrieve submission data.
   *
   * @param string $webform_id
   *   Webform ID.
   *
   * @param int $sid
   *   Submission ID.
   *
   * @return \Drupal\rest\ModifiedResourceResponse
   *   HTTP response object containing webform submission.
   *
   * @throws \Symfony\Component\HttpKernel\Exception\HttpException
   *   Throws HttpException in case of error.
   */
  public function get($webform_id, $sid) {

    if (empty($webform_id) || empty($sid)) {
      $errors = [
        'error' => [
          'message' => 'Both webform ID and submission ID are required.'
        ]
      ];
      return new ModifiedResourceResponse($errors, 400);
    }

    // Load the webform submission.
    /** @var \Drupal\webform\WebformSubmissionInterface $webform_submission */
    $webform_submission = WebformSubmission::load($sid);

    // Check for a submission.
    if (!empty($webform_submission)) {
      $submission_webform_id = $webform_submission->get('webform_id')->getString();

      // Check webform_id.
      if ($submission_webform_id == $webform_id) {

        // Build our response.
        $response = $this->buildResponse($webform_submission);

        // Return the submission.
        return new ModifiedResourceResponse($response);
      }
    }

    throw new NotFoundHttpException(t("Can't load webform submission."));

  }

  /**
   * Update submission data.
   *
   * @param string $webform_id
   *   Webform ID.
   *
   * @param int $sid
   *   Submission ID.
   *
   * @param array $webform_data
   *   Webform field data.
   *
   * @return \Drupal\rest\ModifiedResourceResponse
   *   HTTP response object containing webform submission.
   *
   * @throws \Symfony\Component\HttpKernel\Exception\HttpException
   * @throws \Drupal\Core\Entity\EntityStorageException
   */
  public function patch($webform_id, $sid, array $webform_data) {

    if (empty($webform_id) || empty($sid)) {
      $errors = [
        'error' => [
          'message' => 'Both webform ID and submission ID are required.'
        ]
      ];
      return new ModifiedResourceResponse($errors, 400);
    }

    if (empty($webform_data)) {
      $errors = [
        'error' => [
          'message' => 'No data has been submitted.'
        ]
      ];
      return new ModifiedResourceResponse($errors, 400);
    }

    // Load the webform submission.
    /** @var \Drupal\webform\WebformSubmissionInterface $webform_submission */
    $webform_submission = WebformSubmission::load($sid);

    // Check for a submission.
    if (!empty($webform_submission)) {
      $submission_webform_id = $webform_submission->get('webform_id')->getString();

      // Check webform_id.
      if ($submission_webform_id == $webform_id) {
        foreach ($webform_data as $field => $value) {
          $webform_submission->setElementData($field, $value);
        }

        // Submit the data.
        $webform_submission->save();

        // Build our response.
        $response = [
          'sid' => $webform_submission->id(),
          'entity' => $webform_submission,
          'data' => $webform_submission->getData(),
        ];

        // Return submission ID.
        return new ModifiedResourceResponse($response);
      }
    }

    throw new NotFoundHttpException(t("Can't load webform submission."));
  }

  /**
   * Builder for our typical response.
   *
   * @param \Drupal\webform\WebformSubmissionInterface $webform_submission
   *   Webform submission interface we're working with.
   *
   * @return array
   *   The response array.
   */
  private function buildResponse(WebformSubmissionInterface $webform_submission) {
    // Grab submission data.
    $data = $webform_submission->getData();

    return [
      'sid' => $webform_submission->id(),
      'entity' => $webform_submission,
      'data' => $data
    ];
  }

}
